@@ -6,6 +6,7 @@ import asyncio
 import time
 import datetime
 from datetime import timezone
+import jwt
 
 logger = logging.getLogger("core.task")
 
@@ -31,6 +32,7 @@ class Core:
         self, 
         AGENT_NAME: str, 
         CORE_API: str, 
+        JWT_TOKEN: str | None = None,
     ):
         """
         Initialize Core API handler with agent credentials and base API URL.
@@ -38,6 +40,8 @@ class Core:
         Args:
             AGENT_NAME (str): The agent's username for authentication.
             CORE_API (str): Base URL of the Core API service.
+            JWT_TOKEN (str | None): If provided, skips the need for login() before accessing protected APIs.
+
 
         Raises:
             ValueError: If AGENT_NAME or CORE_API is missing or empty.
@@ -50,9 +54,73 @@ class Core:
 
         self.CORE_API = CORE_API.rstrip("/")
         self.AGENT_NAME = AGENT_NAME
-        self.JWT_TOKEN = None
+        self.JWT_TOKEN = JWT_TOKEN
+        self._PASSWORD: str = None  # Store password for future use
         self.RUNTIME_VARIABLES: dict[str, str | None] = {}
     
+    def is_token_valid(self) -> bool:
+        """
+        Checks if the current JWT token is valid and not expired.
+
+        Returns:
+            bool: True if the token exists and is not expired, False otherwise.
+        """
+        if not self.JWT_TOKEN:
+            return False
+        try:
+            payload = jwt.decode(self.JWT_TOKEN, options={"verify_signature": False})
+            exp = payload.get("exp")
+            if not exp:
+                return False
+            expiry_time = datetime.fromtimestamp(exp, timezone.utc)
+            return expiry_time > datetime.now(timezone.utc)
+        except Exception:
+            return False
+        
+    @classmethod
+    def from_env(cls) -> "Core":
+        """
+        Constructs a Core instance from environment variables.
+        Handles JWT reuse and automatic re-login if token is expired.
+        Returns:
+            Core: Initialized and authenticated Core instance.
+
+        Raises:
+            ValueError: If required environment variables are missing.
+        """
+        agent_name = os.environ.get("AGENT_NAME")
+        core_api = os.environ.get("CORE_API")
+        jwt_token = os.environ.get("AGENTIX_CORE_JWT_TOKEN")
+        password = os.environ.get("CORE_PASSWORD")
+
+        if not agent_name:
+            raise ValueError("Environment variable 'AGENT_NAME' is required but not set.")
+        if not core_api:
+            raise ValueError("Environment variable 'CORE_API' is required but not set.")
+
+        core = cls(
+            AGENT_NAME=agent_name,
+            CORE_API=core_api,
+            JWT_TOKEN=jwt_token,
+        )
+        core._PASSWORD = password
+
+        if not core.is_token_valid():
+            print ("[CORE] Token Expired! Trying to relogin ...")
+            if core._PASSWORD:
+                asyncio.run(core.login(core._PASSWORD))
+                print("[CORE] Re-login successful, JWT token updated.")
+                os.environ["AGENTIX_CORE_JWT_TOKEN"] = core.JWT_TOKEN  # Optional re-cache
+            else:
+                raise RuntimeError("JWT expired and no password available to re-login")
+        else:
+            print("[CORE] JWT Token is valid, no need to re-login.")
+
+        print(f"[CORE] Using JWT Token ... Fetching runtime variables for agent: {core.AGENT_NAME}")
+        asyncio.run(core.fetch_runtime_variables())
+        print(f"[CORE] ✅ Ready")
+        return core
+        
     #================================================================================================
     # Get Runtime Variable by Key
     #================================================================================================
@@ -74,6 +142,7 @@ class Core:
     #================================================================================================
     # Fetch Runtime Variables from Core
     #================================================================================================
+    #TODO: Decorator of ensure Token here
     async def fetch_runtime_variables(self):
         """
         Fetches runtime variables from Core backend and stores them in self.RUNTIME_VARIABLES.
@@ -150,6 +219,7 @@ class Core:
                         login_json = await login_response.json()
                         if login_response.status == 200 and "jwt" in login_json:
                             self.JWT_TOKEN = login_json["jwt"]
+                            self._PASSWORD = PASSWORD  # Store password for future use
                             logger.info(f"✅ [LOGIN] Login succeeded on attempt {attempt}")                            
                             break
                         else:
@@ -173,6 +243,7 @@ class Core:
     #================================================================================================
     # Connect to Task in Core
     #================================================================================================
+    #TODO: Decorator of ensure Token here
     async def connect_task(self, task_key: str) -> dict:
         """
         Establishes a session for the given task and returns job cookies.
@@ -515,6 +586,7 @@ class Core:
     #================================================================================================
     # Create New Job in Core
     #================================================================================================
+    #TODO: Decorator of ensure Token here
     async def new_job(self, workflow_id: int, customer: dict, language: str = "en", valid_until: int = 60, metadata: str = None, external_ref_number: str = None, input_data: list = None) -> dict:
         """
         Creates a new job in the Core system using the specified workflow and customer info.
